import type {SearchOptions} from '../support/helpers';
import {findGroupRow, submitGroupSearch} from '../support/helpers';

const school = 'school1';
const schoolClasses = [
  {
    name: 'class-1',
    displayName: 'class 1',
  },
  {
    name: 'class-2',
    displayName: 'class 2',
  },
  {
    name: 'class-3',
    displayName: 'class 3',
  },
  {
    name: 'class-4',
    displayName: 'class 4',
  },
];

const getKelvinToken = (): Cypress.Chainable<Promise<string>> => cy.request({
  method: 'POST',
  url: 'http://backup1.school.test/ucsschool/kelvin/token',
  form: true,
  body: {
    // eslint-disable-next-line camelcase
    grant_type: 'password',
    username: 'Administrator',
    password: 'univention',
  },
}).then(response => response.body.access_token);

const deleteTestClasses = (): Cypress.Chainable<Promise<string>> => getKelvinToken().then(token => {
  for (const claz of [
    ...schoolClasses,
    {
      name: 'ChangedName',
      displayName: 'class-4',
    },
  ]) {
    cy.request({
      method: 'DELETE',
      url: `http://backup1.school.test/ucsschool/kelvin/v1/classes/${school}/${claz.name}`,
      headers: {
        Authorization: `Bearer ${token}`,
      },
      failOnStatusCode: false,
    });
  }
});

before(() => {
  deleteTestClasses().then(() => {
    getKelvinToken().then(token => {
      for (const claz of schoolClasses) {
        cy.request({
          method: 'POST',
          url: 'http://backup1.school.test/ucsschool/kelvin/v1/classes/',
          headers: {
            Authorization: `Bearer ${token}`,
          },
          body: {
            // eslint-disable-next-line camelcase
            udm_properties: {},
            name: claz.name,
            school: `https://backup1.school.test/ucsschool/kelvin/v1/schools/${school}`,
            description: claz.displayName,
            users: [],
            // eslint-disable-next-line camelcase
            create_share: false,
          },
        });
      }
    });
  });
});


after(() => {
  deleteTestClasses();
});

describe('school class handling tests', () => {
  it('deletes a school class correctly', () => {
    cy.login();
    const searchOptions: SearchOptions = {
      schoolSelection: school,
      quickSearch: schoolClasses[0].name,
    };
    submitGroupSearch(searchOptions);
    const tr = findGroupRow(schoolClasses[0], {assertExists: true});
    tr.find('td:first-child input').click({scrollBehavior: 'center'});

    cy.intercept('DELETE', `/ucsschool/bff-groups/v1/school_class/${school}-${schoolClasses[0].name}`).as('delete');
    cy.contains('.uGrid__header .uButton', 'Löschen')
      .click();
    cy.contains('.uConfirmDialog .uButton', 'Löschen')
      .click();
    cy.wait('@delete');
    cy.get('.uGrid__tableWrapper tbody tr')
      .should('have.length', 0);
    submitGroupSearch(searchOptions);
    cy.get('.uGrid__tableWrapper tbody tr')
      .should('have.length', 0);
  });

  it('deletes multiple school classes correctly', () => {
    cy.login();
    const searchOptions: SearchOptions = {schoolSelection: school};
    submitGroupSearch(searchOptions);
    findGroupRow(schoolClasses[1], {assertExists: true}).then($row => {
      cy.wrap($row).find('td:first-child input').click();
    });
    findGroupRow(schoolClasses[2], {assertExists: true}).then($row => {
      cy.wrap($row).find('td:first-child input').click();
    });

    cy.intercept('DELETE', `/ucsschool/bff-groups/v1/school_class/${school}-${schoolClasses[1].name}`).as('delete1');
    cy.intercept('DELETE', `/ucsschool/bff-groups/v1/school_class/${school}-${schoolClasses[2].name}`).as('delete2');
    cy.contains('.uGrid__header .uButton', 'Löschen')
      .click();
    cy.contains('.uConfirmDialog .uButton', 'Löschen')
      .click();
    cy.wait(['@delete1', '@delete2']);

    submitGroupSearch(searchOptions);
    findGroupRow(schoolClasses[1]).then($row => {
      cy.wrap($row).should('have.length', 0);
    });
    findGroupRow(schoolClasses[2]).then($row => {
      cy.wrap($row).should('have.length', 0);
    });
  });


  it('edits school class correctly', () => {
    cy.login();
    const searchOptions: SearchOptions = {schoolSelection: school};
    submitGroupSearch(searchOptions);
    findGroupRow(schoolClasses[3], {assertExists: true}).then($row => {
      cy.wrap($row).find('td:first-child input').click();
    });

    cy.intercept('GET', `/ucsschool/bff-groups/v1/school_class/${school}-${schoolClasses[3].name}`).as('fetch');
    cy.contains('.uGrid__header .uButton', 'Bearbeiten')
      .click();
    cy.wait('@fetch');
    cy.url().should('contain', `/univention/ui/ucsschool-groups/schoolClass/edit/${school}-${schoolClasses[3].name}`);

    cy.get('input[name="name"]')
      .clear({scrollBehavior: 'center'})
      .type('ChangedName', {scrollBehavior: 'center'});

    cy.intercept('PATCH', `/ucsschool/bff-groups/v1/school_class/${school}-${schoolClasses[3].name}`).as('patch');
    cy.contains('.listView__header__buttons .uButton--primary', 'Speichern')
      .click();
    cy.wait('@patch');
    submitGroupSearch(searchOptions);
  });
});
