import type {SearchOptions} from '../support/helpers';
import {createMultipleWorkGroups, createWorkGroup, findGroupRow, submitGroupSearch} from '../support/helpers';
import {randomGroupData} from '../support/rand';

const school = 'school1';

describe('Work group handling tests', () => {
  it('creates and deletes a work group correctly', () => {
    cy.login();
    const workGroup = createWorkGroup(school);
    const searchOptions: SearchOptions = {
      schoolSelection: school,
      quickSearch: workGroup.name,
    };
    submitGroupSearch(searchOptions);
    const tr = findGroupRow(workGroup, {assertExists: true});
    tr.find('td:first-child input').click();

    cy.intercept('DELETE', `/ucsschool/bff-groups/v1/workgroup/${school}-${workGroup.name}`).as('delete');
    cy.contains('.uGrid__header .uButton', 'Löschen')
      .click();
    cy.contains('.uConfirmDialog .uButton', 'Löschen')
      .click();
    cy.wait('@delete');
    cy.get('.uGrid__tableWrapper tbody tr')
      .should('have.length', 0);
    submitGroupSearch(searchOptions);
    cy.get('.uGrid__tableWrapper tbody tr')
      .should('have.length', 0);
  });

  it('creates and edits work group correctly', () => {
    cy.login();
    const originalWorkGroupData = randomGroupData();
    const editedWorkGroupData = randomGroupData();
    createWorkGroup(school, originalWorkGroupData);
    const searchOptions: SearchOptions = {schoolSelection: school};
    submitGroupSearch(searchOptions);
    findGroupRow(originalWorkGroupData, {assertExists: true})
      .find('td:first-child input')
      .click();
    cy.intercept('GET', `/ucsschool/bff-groups/v1/workgroup/${school}-${originalWorkGroupData.name}`).as('fetch');
    cy.contains('.uGrid__header .uButton', 'Bearbeiten')
      .click();
    cy.wait('@fetch');
    cy.url().should('contain', `/univention/ui/ucsschool-groups/workgroup/edit/${school}-${originalWorkGroupData.name}`);
    cy.get('input[name="description"]')
      .clear({scrollBehavior: 'center'})
      .type(editedWorkGroupData.name, {scrollBehavior: 'center'});
    cy.get('input[name="name"]')
      .clear({scrollBehavior: 'center'})
      .type(editedWorkGroupData.name, {scrollBehavior: 'center'});
    cy.intercept('PATCH', `/ucsschool/bff-groups/v1/workgroup/${school}-${originalWorkGroupData.name}`).as('patch');
    cy.contains('.listView__header__buttons .uButton--primary', 'Speichern')
      .click();
    cy.wait('@patch');
    submitGroupSearch(searchOptions);
    findGroupRow(originalWorkGroupData).then($row => {
      cy.wrap($row).should('have.length', 0);
    });
    findGroupRow(editedWorkGroupData, {assertExists: true}).then($row => {
      if (editedWorkGroupData.name) {
        cy.wrap($row.find('td:nth-child(4)')).should('contain.text', editedWorkGroupData.name);
      }
    });
  });

  it('creates workgroup and edits workgroups lms state and search for it correctly', () => {
    cy.login();
    const originalWorkGroupData = randomGroupData();
    createWorkGroup(school, originalWorkGroupData);
    const searchOptions: SearchOptions = {schoolSelection: school};
    submitGroupSearch(searchOptions);
    findGroupRow(originalWorkGroupData, {assertExists: true}).then($row => {
      cy.wrap($row.find('td:nth-child(14)')).should('contain.text', 'Deaktiviert');
    });
    findGroupRow(originalWorkGroupData, {assertExists: true})
      .find('td:first-child input')
      .click();
    cy.intercept('GET', `/ucsschool/bff-groups/v1/workgroup/${school}-${originalWorkGroupData.name}`).as('fetch');
    cy.contains('.uGrid__header .uButton', 'Bearbeiten')
      .click();
    cy.wait('@fetch');
    cy.url().should('contain', `/univention/ui/ucsschool-groups/workgroup/edit/${school}-${originalWorkGroupData.name}`);
    cy.get('input[name="description"]')
      .clear({scrollBehavior: 'center'})
      .type(originalWorkGroupData.name, {scrollBehavior: 'center'});
    cy.get('input[name="isLmsGroup"]')
      .check();
    cy.intercept('PATCH', `/ucsschool/bff-groups/v1/workgroup/${school}-${originalWorkGroupData.name}`).as('patch');
    cy.contains('.listView__header__buttons .uButton--primary', 'Speichern')
      .click();
    cy.wait('@patch');
    searchOptions.isLmsGroup = true;
    submitGroupSearch(searchOptions);
    findGroupRow(originalWorkGroupData).then($row => {
      cy.wrap($row).should('have.length', 1);
    });
    findGroupRow(originalWorkGroupData, {assertExists: true}).then($row => {
      if (originalWorkGroupData.name) {
        cy.wrap($row.find('td:nth-child(4)')).should('contain.text', originalWorkGroupData.name);
      }
      cy.wrap($row.find('td:nth-child(14)')).should('contain.text', 'LMS aktiviert');
    });
  });

  it('creates workgroup and edits workgroups Iserv state and search for it correctly', () => {
    cy.login();
    const originalWorkGroupData = randomGroupData();
    createWorkGroup(school, originalWorkGroupData);
    const searchOptions: SearchOptions = {schoolSelection: school};
    submitGroupSearch(searchOptions);
    findGroupRow(originalWorkGroupData, {assertExists: true}).then($row => {
      cy.wrap($row.find('td:nth-child(15)')).should('contain.text', 'IServ deaktiviert');
    });
    findGroupRow(originalWorkGroupData, {assertExists: true})
      .find('td:first-child input')
      .click();
    cy.intercept('GET', `/ucsschool/bff-groups/v1/workgroup/${school}-${originalWorkGroupData.name}`).as('fetch');
    cy.contains('.uGrid__header .uButton', 'Bearbeiten')
      .click();
    cy.wait('@fetch');
    cy.url().should('contain', `/univention/ui/ucsschool-groups/workgroup/edit/${school}-${originalWorkGroupData.name}`);
    cy.get('input[name="description"]')
      .clear({scrollBehavior: 'center'})
      .type(originalWorkGroupData.name, {scrollBehavior: 'center'});
    cy.get('input[name="isIservGroup"]')
      .check();
    cy.intercept('PATCH', `/ucsschool/bff-groups/v1/workgroup/${school}-${originalWorkGroupData.name}`).as('patch');
    cy.contains('.listView__header__buttons .uButton--primary', 'Speichern')
      .click();
    cy.wait('@patch');
    searchOptions.isIservGroup = true;
    submitGroupSearch(searchOptions);
    findGroupRow(originalWorkGroupData).then($row => {
      cy.wrap($row).should('have.length', 1);
    });
    findGroupRow(originalWorkGroupData, {assertExists: true}).then($row => {
      if (originalWorkGroupData.name) {
        cy.wrap($row.find('td:nth-child(4)')).should('contain.text', originalWorkGroupData.name);
      }
      cy.wrap($row.find('td:nth-child(15)')).should('contain.text', 'Aktiviert');
    });
  });

  it('creates and deletes multiple work groups correctly', () => {
    cy.login();
    const workGroups = createMultipleWorkGroups(school, 2);
    const searchOptions: SearchOptions = {schoolSelection: school};
    submitGroupSearch(searchOptions);
    findGroupRow(workGroups[0], {assertExists: true}).then($row => {
      cy.wrap($row).find('td:first-child input').click();
    });
    findGroupRow(workGroups[1], {assertExists: true}).then($row => {
      cy.wrap($row).find('td:first-child input').click();
    });

    cy.intercept('DELETE', `/ucsschool/bff-groups/v1/workgroup/${school}-${workGroups[0].name}`).as('delete1');
    cy.intercept('DELETE', `/ucsschool/bff-groups/v1/workgroup/${school}-${workGroups[1].name}`).as('delete2');
    cy.contains('.uGrid__header .uButton', 'Löschen')
      .click();
    cy.contains('.uConfirmDialog .uButton', 'Löschen')
      .click();
    cy.wait(['@delete1', '@delete2']);

    submitGroupSearch(searchOptions);
    findGroupRow(workGroups[0]).then($row => {
      cy.wrap($row).should('have.length', 0);
    });
    findGroupRow(workGroups[1]).then($row => {
      cy.wrap($row).should('have.length', 0);
    });
  });
});
